@@ -19,7 +19,7 @@ from __future__ import unicode_literals
 import re
 import time
 import willie
-from willie.tools import Nick, iteritems
+from willie.tools import Identifier, iteritems
 import base64
 from willie.logger import get_logger
 
@@ -47,18 +47,12 @@ def startup(bot, trigger):
     bot.connection_registered = True
 
     if bot.config.core.nickserv_password is not None:
+        nickserv_name = bot.config.core.nickserv_name or 'NickServ'
         bot.msg(
-            'NickServ',
+            nickserv_name,
             'IDENTIFY %s' % bot.config.core.nickserv_password
         )
 
-    if (bot.config.core.oper_name is not None
-            and bot.config.core.oper_password is not None):
-        bot.write((
-            'OPER',
-            bot.config.core.oper_name + ' ' + bot.config.oper_password
-        ))
-
     #Use Authserv if authserv_password and authserv_account is set in config.
     if (bot.config.core.authserv_password is not None
             and bot.config.core.authserv_account is not None):
@@ -126,10 +120,10 @@ def handle_names(bot, trigger):
     names = trigger.split()
 
     #TODO specific to one channel type. See issue 281.
-    channels = re.search('(#\S*)', bot.raw)
+    channels = re.search('(#\S*)', trigger.raw)
     if not channels:
         return
-    channel = Nick(channels.group(1))
+    channel = Identifier(channels.group(1))
     if channel not in bot.privileges:
         bot.privileges[channel] = dict()
     bot.init_ops_list(channel)
@@ -147,7 +141,7 @@ def handle_names(bot, trigger):
         for prefix, value in iteritems(mapping):
             if prefix in name:
                 priv = priv | value
-        nick = Nick(name.lstrip(''.join(mapping.keys())))
+        nick = Identifier(name.lstrip(''.join(mapping.keys())))
         bot.privileges[channel][nick] = priv
 
         # Old op list maintenance is down here, and should be removed at some
@@ -171,7 +165,7 @@ def handle_names(bot, trigger):
 def track_modes(bot, trigger):
     """Track usermode changes and keep our lists of ops up to date."""
     # Mode message format: <channel> *( ( "-" / "+" ) *<modes> *<modeparams> )
-    channel = Nick(trigger.args[0])
+    channel = Identifier(trigger.args[0])
     line = trigger.args[1:]
 
     # If the first character of where the mode is being set isn't a #
@@ -179,24 +173,6 @@ def track_modes(bot, trigger):
     if channel.is_nick():
         return
 
-    def handle_old_modes(nick, mode):
-        #Old mode maintenance. Drop this crap in 5.0.
-        if mode[1] == 'o' or mode[1] == 'q' or mode[1] == 'a':
-            if mode[0] == '+':
-                bot.add_op(channel, nick)
-            else:
-                bot.del_op(channel, nick)
-        elif mode[1] == 'h':  # Halfop
-            if mode[0] == '+':
-                bot.add_halfop(channel, nick)
-            else:
-                bot.del_halfop(channel, nick)
-        elif mode[1] == 'v':
-            if mode[0] == '+':
-                bot.add_voice(channel, nick)
-            else:
-                bot.del_voice(channel, nick)
-
     mapping = {'v': willie.module.VOICE,
                'h': willie.module.HALFOP,
                'o': willie.module.OP,
@@ -218,7 +194,7 @@ def track_modes(bot, trigger):
                 else:
                     modes.append(sign + char)
         else:
-            arg = Nick(arg)
+            arg = Identifier(arg)
             for mode in modes:
                 priv = bot.privileges[channel].get(arg, 0)
                 value = mapping.get(mode[1])
@@ -228,7 +204,6 @@ def track_modes(bot, trigger):
                     else:
                         priv = priv & ~value
                     bot.privileges[channel][arg] = priv
-                handle_old_modes(arg, mode)
 
 
 @willie.module.rule('.*')
@@ -239,27 +214,27 @@ def track_modes(bot, trigger):
 def track_nicks(bot, trigger):
     """Track nickname changes and maintain our chanops list accordingly."""
     old = trigger.nick
-    new = Nick(trigger)
+    new = Identifier(trigger)
 
     # Give debug mssage, and PM the owner, if the bot's own nick changes.
     if old == bot.nick:
-        privmsg = "Hi, I'm your bot, %s." + \
-            " Something has made my nick change." + \
-            " This can cause some problems for me," + \
-            " and make me do weird things." + \
-            " You'll probably want to restart me," + \
-            " and figure out what made that happen" + \
-            " so you can stop it happening again." + \
-            " (Usually, it means you tried to give me a nick" + \
-            " that's protected by NickServ.)" % bot.nick
-        debug_msg = "Nick changed by server." + \
-            " This can cause unexpected behavior. Please restart the bot."
+        privmsg = ("Hi, I'm your bot, %s."
+                   "Something has made my nick change. "
+                   "This can cause some problems for me, "
+                   "and make me do weird things. "
+                   "You'll probably want to restart me, "
+                   "and figure out what made that happen "
+                   "so you can stop it happening again. "
+                   "(Usually, it means you tried to give me a nick "
+                   "that's protected by NickServ.)") % bot.nick
+        debug_msg = ("Nick changed by server. "
+            "This can cause unexpected behavior. Please restart the bot.")
         LOGGER.critical(debug_msg)
         bot.msg(bot.config.core.owner, privmsg)
         return
 
     for channel in bot.privileges:
-        channel = Nick(channel)
+        channel = Identifier(channel)
         if old in bot.privileges[channel]:
             value = bot.privileges[channel].pop(old)
             bot.privileges[channel][new] = value
@@ -301,7 +276,7 @@ def track_part(bot, trigger):
 @willie.module.thread(False)
 @willie.module.unblockable
 def track_kick(bot, trigger):
-    nick = Nick(trigger.args[1])
+    nick = Identifier(trigger.args[1])
     if nick == bot.nick:
         bot.channels.remove(trigger.sender)
         del bot.privileges[trigger.sender]
@@ -429,7 +404,7 @@ def auth_proceed(bot, trigger):
     sasl_token = '\0'.join((sasl_username, sasl_username,
                            bot.config.core.sasl_password))
     # Spec says we do a base 64 encode on the SASL stuff
-    bot.write(('AUTHENTICATE', base64.b64encode(sasl_token)))
+    bot.write(('AUTHENTICATE', base64.b64encode(sasl_token.encode('utf-8'))))
 
 
 @willie.module.event('903')
@@ -466,7 +441,7 @@ def blocks(bot, trigger):
     }
 
     masks = bot.config.core.get_list('host_blocks')
-    nicks = [Nick(nick) for nick in bot.config.core.get_list('nick_blocks')]
+    nicks = [Identifier(nick) for nick in bot.config.core.get_list('nick_blocks')]
     text = trigger.group().split()
 
     if len(text) == 3 and text[1] == "list":
@@ -503,10 +478,10 @@ def blocks(bot, trigger):
 
     elif len(text) == 4 and text[1] == "del":
         if text[2] == "nick":
-            if Nick(text[3]) not in nicks:
+            if Identifier(text[3]) not in nicks:
                 bot.reply(STRINGS['no_nick'] % (text[3]))
                 return
-            nicks.remove(Nick(text[3]))
+            nicks.remove(Identifier(text[3]))
             bot.config.core.nick_blocks = nicks
             bot.config.save()
             bot.reply(STRINGS['success_del'] % (text[3]))
