@@ -278,21 +278,6 @@ def get_command_regexp(prefix, command):
     return re.compile(pattern, re.IGNORECASE | re.VERBOSE)
 
 
-def deprecate_for_5(thing):
-    warnings.warn(thing + ' will be removed in Willie 5.0. Please see '
-                  'http://willie.dftba.net/willie_5.html for more info.',
-                  stacklevel=3)
-
-
-def deprecated_5(old):
-    def new(*args, **kwargs):
-        deprecate_for_5(old.__name__)
-        return old(*args, **kwargs)
-    new.__doc__ = old.__doc__
-    new.__name__ = old.__name__
-    return new
-
-
 def deprecated(old):
     def new(*args, **kwargs):
         print('Function %s is deprecated.' % old.__name__, file=sys.stderr)
@@ -428,10 +413,6 @@ class Identifier(unicode):
         return self and not self.startswith(_channel_prefixes)
 
 
-Nick = Identifier
-"""Identical to the Identifier class. Deprecated from 4.6.0."""
-
-
 class OutputRedirect:
 
     """Redirect te output to the terminal and a log file.
@@ -473,6 +454,12 @@ class OutputRedirect:
             except UnicodeDecodeError:
                 # we got an invalid string, safely encode it to utf-8
                 logfile.write(unicode(string, 'utf8', errors="replace"))
+    
+    def flush(self):
+        if self.stderr:
+            sys.__stderr__.flush()
+        else:
+            sys.__stdout__.flush()
 
 
 #These seems to trace back to when we thought we needed a try/except on prints,
@@ -513,9 +500,10 @@ def get_timezone(db=None, config=None, zone=None, nick=None, channel=None):
 
     Time zone is pulled in the following priority:
     1. `zone`, if it is valid
-    2. The timezone for `zone` in `db` if one is set and valid.
-    3. The timezone for `nick` in `db`, if one is set and valid.
-    4. The timezone for `channel` in `db`, if one is set and valid.
+    2. The timezone for the channel or nick `zone` in `db` if one is set and
+       valid.
+    3. The timezone for the nick `nick` in `db`, if one is set and valid.
+    4. The timezone for the channel  `channel` in `db`, if one is set and valid.
     5. The default timezone in `config`, if one is set and valid.
 
     If `db` is not given, or given but not set up, steps 2 and 3 will be
@@ -551,12 +539,12 @@ def get_timezone(db=None, config=None, zone=None, nick=None, channel=None):
 
     if zone:
         tz = check(zone)
-        if not tz and zone in db.preferences:
-            tz = check(db.preferences.get(zone, 'tz'))
-    if not tz and nick and nick in db.preferences:
-        tz = check(db.preferences.get(nick, 'tz'))
-    if not tz and channel and channel in db.preferences:
-        tz = check(db.preferences.get(channel, 'tz'))
+        if not tz:
+            tz = check(db.get_nick_or_channel_value(zone, 'timezone'))
+    if not tz and nick:
+        tz = check(db.get_nick_value(nick, 'timezone'))
+    if not tz and channel:
+        tz = check(db.get_channel_value(channel, 'timezone'))
     if not tz and config and config.has_option('core', 'default_timezone'):
         tz = check(config.core.default_timezone)
     return tz
@@ -575,8 +563,8 @@ def format_time(db=None, config=None, zone=None, nick=None, channel=None,
 
     The format for the string is chosen in the following order:
 
-    1. The format for `nick` in `db`, if one is set and valid.
-    2. The format for `channel` in `db`, if one is set and valid.
+    1. The format for the nick `nick` in `db`, if one is set and valid.
+    2. The format for the channel `channel` in `db`, if one is set and valid.
     3. The default format in `config`, if one is set and valid.
     4. ISO-8601
 
@@ -584,10 +572,10 @@ def format_time(db=None, config=None, zone=None, nick=None, channel=None,
     is not given, step 3 will be skipped."""
     tformat = None
     if db:
-        if nick and nick in db.preferences:
-            tformat = db.preferences.get(nick, 'time_format')
-        if not tformat and channel in db.preferences:
-            tformat = db.preferences.get(channel, 'time_format')
+        if nick:
+            tformat = db.get_nick_value(nick, 'time_format')
+        if not tformat and channel:
+            tformat = db.get_channel_value(channel, 'time_format')
     if not tformat and config and config.has_option('core',
                                                     'default_time_format'):
         tformat = config.core.default_time_format
